﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace _3BloxPlayer
{
    public partial class Form1 : Form
    {
        private static readonly Timer _timer = new Timer();
        private MouseHook _mouseHook;

        private Bitmap _bitmap;
        private Graphics _graphics;

        private int _width;
        private int _height;
        private int _left;
        private int _top;

        private int _mouseX;
        private int _mouseY;

        private int R { get { return 150; } }

        public Form1()
        {
            InitializeComponent();

            TopMost = true;

            _timer.Tick += new EventHandler(TimerTick);
            _timer.Interval = 100;
        }

        private void BtnAuto_Click(object sender, EventArgs e)
        {
            ReadTextBoxes();

            var rand = new Random();

            var bitmap = new Bitmap(_width, _height, PixelFormat.Format32bppArgb);
            var graphics = Graphics.FromImage(bitmap);

            MoveMouseToStartButton();

            lblInfo.Text = "Started";

            List<int> columns = new List<int>{95, 122, 149};
            var columnCount = columns.Count();

            for (int i = (_height - 13); i > 0; i -= 27)
            {
                var result = true;
                do
                {
                    graphics.CopyFromScreen(_left, _top, 0, 0, bitmap.Size);

                } while (!BlocksExistAtColumns(bitmap, i, columns));

                Mouse.SendSingleClick();

                graphics.CopyFromScreen(_left, _top, 0, 0, bitmap.Size);
                bitmap.Save(string.Format("Screen_{0}.bmp", DateTime.Now.ToString("yyyyMMHHmmss")), ImageFormat.Bmp);

                System.Threading.Thread.Sleep(rand.Next(1000, 2000));

                graphics.CopyFromScreen(_left, _top, 0, 0, bitmap.Size);
                columns = GetBlockColumns(bitmap, i).ToList();
                columnCount = columns.Count();
                if (columnCount < 1)
                {
                    break;
                }

                lblInfo.Text = string.Format("Click {0}", i);
            }

            bitmap.Save(string.Format("Screen_{0}.bmp", DateTime.Now.ToString("yyyyMMHHmmss")), ImageFormat.Bmp);

            bitmap.Dispose();
            lblInfo.Text = "Finished";
        }

        private void MoveMouseToStartButton()
        {
            Cursor.Position = new Point(_mouseX, _mouseY);
        }

        private IEnumerable<int> GetBlockColumns(Bitmap bitmap, int height)
        {
            var columns = new List<int>();
            for (var i = 13; i < _width; i += 27)
            {
                if (bitmap.GetPixel(i, height).R > R)
                    columns.Add(i);
                else if (columns.Count > 1)
                    break;

            }
            return columns;
        }

        private bool BlocksExistAtColumns(Bitmap bitmap, int height, IEnumerable<int> columns)
        {
            var result = true;
            foreach (var column in columns)
            {
                result = result && bitmap.GetPixel(column, height).R > R;
            }
            return result;
        }

        private void BtnMouse_Click(object sender, EventArgs e)
        {
            var button = (Button) sender;

            if (!_timer.Enabled)
            {
                _timer.Start();
                button.Text = "On";
            }
            else
            {
                _timer.Stop();
                button.Text = "Off";
            }
        }

        private void TimerTick(object sender, EventArgs e)
        {
            lblLeft.Text = string.Format("X: {0}", Cursor.Position.X);
            lblTop.Text = string.Format("Y: {0}", Cursor.Position.Y);
        }

        private void ReadTextBoxes()
        {
            _width = int.Parse(txtWidth.Text);
            _height = int.Parse(txtHight.Text);
            _left = int.Parse(txtLeft.Text);
            _top = int.Parse(txtTop.Text);
            _mouseX = int.Parse(txtMouseX.Text);
            _mouseY = int.Parse(txtMouseY.Text);
        }

        private void BtnBenchMark_Click(object sender, EventArgs e)
        {
            ReadTextBoxes();

            var bitmap = new Bitmap(_width, _height, PixelFormat.Format32bppArgb);
            var graphics = Graphics.FromImage(bitmap);

            var start = DateTime.UtcNow;
            for(int i = 0; i < 1000; i++)
            {
                graphics.CopyFromScreen(_left, _top, 0, 0, bitmap.Size);
            }
            var end = DateTime.UtcNow;

            var elipsed = end.Subtract(start).TotalMilliseconds;
            lblInfo.Text = (elipsed).ToString();
        }

        private void BtnManual_Click(object sender, EventArgs e)
        {
            var button = (Button)sender;

            if (_mouseHook != null && _mouseHook.IsStarted)
            {
                _mouseHook.Stop();
                button.Text = "Manual";
            }
            else
            {
                ReadTextBoxes();

                _bitmap = new Bitmap(_width, _height, PixelFormat.Format32bppArgb);
                _graphics = Graphics.FromImage(_bitmap);

                _mouseHook = new MouseHook();
                _mouseHook.MouseDown += new MouseEventHandler(MouseHook_Click);
                _mouseHook.Start();

                button.Text = "On";
            }
        }

        private void MouseHook_Click(object sender, MouseEventArgs e)
        {
            _graphics.CopyFromScreen(_left, _top, 0, 0, _bitmap.Size);
            _bitmap.Save(string.Format("Click_{0}.bmp", DateTime.Now.ToString("yyyyMMHHmmss")), ImageFormat.Bmp);
        }
    }
}
